﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Reflection;
using UnityEngine;

namespace Mod
{
    public static class TransformDeepChildExtension
    {
        //Breadth-first search
        public static Transform FindDeepChild(this Transform aParent, string aName)
        {
            if (aParent.name == aName)
                return aParent;
            foreach (Transform t in aParent)
                FindDeepChild(t, aName);
            return null;
        }
    }
    public class BloodySyringe : SyringeBehaviour
    {

        public static void Load()
        {
            ModAPI.RegisterLiquid(BloodySyringeSerum.ID, new BloodySyringeSerum());

            ModAPI.Register(
                new Modification()
                {
                    OriginalItem = ModAPI.FindSpawnable("Acid Syringe"),

                    NameOverride = "​<b></b>" + "Purification Syringe",
                    DescriptionOverride = "Syringe that drains all liquids that are not blood and removes their effects.",
                    ThumbnailOverride = ModAPI.LoadSprite("sprites/invc.png"),

                    CategoryOverride = ModAPI.FindCategory("Polars Bear"),
                    AfterSpawn = (Instance) => { Destroy(Instance.GetComponent<SyringeBehaviour>()); Instance.GetOrAddComponent<BloodySyringe>(); }
                }
            );

            Mod.nailgun.SyringeNail("​<b></b>" + "Purification Syringe", typeof(BloodySyringe));
        }

        void Awake()
        {
            base.Awake();
            Liquid liquid = Liquid.GetLiquid(GetLiquidID());
            LightSprite light = ModAPI.CreateLight(transform, liquid.Color, 0.2f, 3f);
            light.gameObject.layer = gameObject.layer;
        }
        public override string GetLiquidID() => BloodySyringeSerum.ID;
        public class BloodySyringeSerum : libs.OneTimeBodyLiquid
        {
            FieldInfo mi = typeof(Liquid).GetField("ID");
            public const string ID = "​<b></b>" + "FLUID RESTORING AGENT";
            public BloodySyringeSerum()
            {
                Color = new UnityEngine.Color(1.00000f, 1.00000f, 1.0000f);
            }
            public override void OnEnterLimb(LimbBehaviour limb)
            {
                SpawnableAsset spawnable = ModAPI.FindSpawnable(limb.Person.gameObject.name);
                Dictionary<string, LimbBehaviour> limbs = new Dictionary<string, LimbBehaviour>();
                foreach(LimbBehaviour limbo in spawnable.Prefab.GetComponentsInChildren<LimbBehaviour>())
                {
                    limbs.Add(limbo.gameObject.name, limbo);
                }
                foreach (LimbBehaviour limby in limb.Person.Limbs)
                {
                    DrainOutsiders drainer = limby.gameObject.GetOrAddComponent<DrainOutsiders>();
                    drainer.ogLimb = limbs[limby.gameObject.name].gameObject;
                    limby.BloodLiquidType = limbs[limby.gameObject.name].BloodLiquidType;
                }
            }

            public class DrainOutsiders : MonoBehaviour
            {
                LimbBehaviour limb;
                CirculationBehaviour circ;
                CustomBleedingParticleBehaviour bleed;
                Liquid thisLiquid;
                float destroyFromNoActivity = 0f;
                public GameObject ogLimb;
                public bool didDrain = false;

                void Awake()
                {
                    limb = GetComponent<LimbBehaviour>();
                    limb.StopAllCoroutines();
                    circ = limb.CirculationBehaviour;
                    circ.StopAllCoroutines();
                    thisLiquid = Liquid.GetLiquid(ID);
                    Tuple<KeyValuePair<Liquid, RefFloat>[], float> t = GetOutsiders(circ, thisLiquid);
                    bleed = AddDrainagePoint(circ);
                    DAMNITDEVS();
                }

                void OnDestroy()
                {
                    limb.Person.StartCoroutine(DestroyAfterLifetime(bleed.gameObject));
                    List<Type> ogComponents = new List<Type>();
                    foreach (Component c in ogLimb.GetComponents<Component>())
                    {
                        ogComponents.Add(c.GetType());
                    }
                    Component[] currentComponents = limb.GetComponents<Component>();
                    foreach (Component c in currentComponents)
                    {
                        if (!ogComponents.Contains(c.GetType()))
                        {
                            Destroy(c);
                        }
                    }
                    limb.Person.StartCoroutine(LerpColor(limb, ogLimb.GetComponent<LimbBehaviour>().Color));
                }

                IEnumerator LerpColor(LimbBehaviour limb, Color target)
                {
                    float progress = 0f;
                    target = new Color(1f, 1f, 1f, 1f);
                    while (limb.Color != target)
                    {
                        progress += Time.deltaTime/2;
                        Debug.Log(progress);
                        limb.Color = UnityEngine.Color.Lerp(limb.Color,target,progress);
                        yield return new WaitForEndOfFrame();
                    }
                }

                void DAMNITDEVS()
                {
                    Debug.Log("STOP BRRREEEEEAKING STUFFF, GAME DEVS!");
                }

                void FixedUpdate()
                {
                    Tuple<KeyValuePair<Liquid, RefFloat>[], float> t = GetOutsiders(circ,thisLiquid);
                    KeyValuePair<Liquid, RefFloat>[] outsiders = t.Item1;
                    float totalOutsiderAmount = t.Item2;
                    if (outsiders.Length == 0) { bleed.gameObject.SetActive(false); destroyFromNoActivity += Time.deltaTime; if (destroyFromNoActivity > 0.1f) { Destroy(this); } return; }
                    else { bleed.gameObject.SetActive(true); destroyFromNoActivity = 0f; didDrain = true; }


                    Color color = Color.black;

                    foreach (KeyValuePair<Liquid, RefFloat> liquid in outsiders)
                    {
                        color += liquid.Key.Color * (liquid.Value.Raw / totalOutsiderAmount);
                        circ.RemoveLiquid(liquid.Key, 0.1f * Time.deltaTime);
                    }
                    circ.DeleteEmptyLiquidEntries();
                    bleed.computedColor = color;
                    bleed.totalLiquidAmount = totalOutsiderAmount;
                }

                IEnumerator DestroyAfterLifetime(GameObject go)
                {
                    ParticleSystem ps = go.GetComponent<ParticleSystem>();
                    ps.Stop();
                    yield return new WaitWhile(delegate { return ps.particleCount > 0; });
                    Destroy(go);
                }

                CustomBleedingParticleBehaviour AddDrainagePoint(CirculationBehaviour circ)
                {
                    Vector3 pos = circ.Limb.transform.position + new Vector3(UnityEngine.Random.Range(-0.1f, 0.1f), UnityEngine.Random.Range(-0.1f, 0.1f));
                    Vector2 worldPosition = circ.Limb.PhysicalBehaviour.spriteRenderer.bounds.ClosestPoint(pos);
                    Vector2 direction = Vector2.MoveTowards(worldPosition, circ.Limb.transform.position, 1f);
                    GameObject gameObject = UnityEngine.Object.Instantiate<GameObject>(circ.Limb.Person.BleedingParticlePrefab, worldPosition - direction * 0.1f, Quaternion.identity, circ.Limb.transform);
                    gameObject.transform.up = direction;
                    BleedingParticleBehaviour component = gameObject.GetComponent<BleedingParticleBehaviour>();
                    CustomBleedingParticleBehaviour bleed = gameObject.AddComponent<CustomBleedingParticleBehaviour>();
                    bleed.BloodSplatController = component.BloodSplatController;
                    bleed.WaterPuffSystem = component.WaterPuffSystem;
                    bleed.ShouldBecomeSmokeInWater = true;
                    Destroy(component);
                    bleed.gameObject.SetActive(false);
                    return bleed;
                }

                public class CustomBleedingParticleBehaviour : MonoBehaviour
                {
                    public float totalLiquidAmount;
                    private void Awake()
                    {
                        this.BloodParticleSystem = base.GetComponent<ParticleSystem>();
                        this.main = this.BloodParticleSystem.main;
                        this.emission = this.BloodParticleSystem.emission;
                    }

                    private void Start()
                    {
                        if (!this.ShouldBecomeSmokeInWater && this.WaterPuffSystem)
                        {
                            UnityEngine.Object.Destroy(this.WaterPuffSystem);
                        }
                    }

                    private void Update()
                    {
                        float num = 1f;
                        bool flag = WaterBehaviour.IsPointUnderWater(base.transform.position);
                        this.main.startSpeedMultiplier = this.SpeedMultiplier * (1f * 1f * 1f * 5f) * UnityEngine.Random.Range(0.14f, 0.2f) * 1f;
                        this.main.startColor = computedColor;
                        float num2 = 1f;
                        this.emission.rateOverTimeMultiplier = (flag ? 0f : (24f * num2 * this.RateMultiplier));
                        this.emission.enabled = true;
                        if (this.BloodSplatController)
                        {
                            this.BloodSplatController.DecalColourMultiplier = computedColor;
                            this.BloodSplatController.EjectionForce = new Vector2(0.25f, 0.35f) * this.main.startSpeedMultiplier;
                            this.BloodSplatController.SpawningChance = (flag ? 0f : totalLiquidAmount);
                        }
                        if (this.ShouldBecomeSmokeInWater && this.WaterPuffSystem)
                        {
                            var main = this.WaterPuffSystem.main;
                            main.startColor = computedColor;
                            if (flag)
                            {
                                var emission = this.WaterPuffSystem.emission;
                                emission.rateOverTimeMultiplier = num2;
                                if (!this.WaterPuffSystem.isPlaying)
                                {
                                    this.WaterPuffSystem.Play();
                                    return;
                                }
                            }
                            else if (this.WaterPuffSystem.isPlaying)
                            {
                                this.WaterPuffSystem.Stop();
                                return;
                            }
                        }
                    }

                    public CirculationBehaviour CirculationBehaviour;

                    public ParticleSystem BloodParticleSystem;

                    public Color computedColor;

                    public ParticleSystem WaterPuffSystem;

                    public CirculationBehaviour PushingTo;

                    public FancyBloodSplatController BloodSplatController;

                    private ParticleSystem.MainModule main;

                    private ParticleSystem.EmissionModule emission;

                    public float SpeedMultiplier = 1f;

                    public float RateMultiplier = 1f;

                    public bool ShouldBecomeSmokeInWater;
                }

                Color GetColorOfLiquids(KeyValuePair<Liquid, BloodContainer.RefFloat>[] pairs)
                {
                    Color color = Color.black;
                    foreach (KeyValuePair<Liquid, BloodContainer.RefFloat> keyValuePair in pairs)
                    {
                        color += keyValuePair.Key.Color * keyValuePair.Value.Raw;
                    }
                    color.a = 1f;
                    color.r = Mathf.Clamp01(color.r);
                    color.g = Mathf.Clamp01(color.g);
                    color.b = Mathf.Clamp01(color.b);
                    return color;
                }

                Tuple<KeyValuePair<Liquid, BloodContainer.RefFloat>[], float> GetOutsiders(CirculationBehaviour circ, Liquid thisLiquid)
                {
                    List<KeyValuePair<Liquid, RefFloat>> pairs = new List<KeyValuePair<Liquid, RefFloat>>();
                    float totalAmount = 0f;
                    foreach (KeyValuePair<Liquid, BloodContainer.RefFloat> keyValuePair in circ.LiquidDistribution)
                    {
                        if (keyValuePair.Key == thisLiquid || keyValuePair.Key == circ.Limb.GetOriginalBloodType()) { continue; }
                        pairs.Add(keyValuePair);
                        totalAmount += keyValuePair.Value.Raw;
                    }
                    return Tuple.Create(pairs.ToArray(), totalAmount);
                }
            }


            

        }

    }

}

//Author: Polars Bear
